@@ -1,7 +1,7 @@
 /*
  * libics: Image Cytometry Standard file reading and writing.
  *
- * Copyright 2015-2017:
+ * Copyright 2015-2019:
  *   Scientific Volume Imaging Holding B.V.
  *   Laapersveld 63, 1213 VB Hilversum, The Netherlands
  *   https://www.svi.nl
@@ -50,12 +50,25 @@ extern "C" {
 #endif
 
 /* Library versioning is in the form major, minor, patch: */
-#define ICSLIB_VERSION "1.6.2" /* also defined in configure.ac */
+#define ICSLIB_VERSION "1.6.3" /* also defined in configure.ac */
 
 #if defined(__WIN32__) && !defined(WIN32)
 #define WIN32
 #endif
 
+#if defined(_WIN64) && !defined(WIN64)
+#define WIN64
+#endif
+    
+
+/* Windows platforms have a different case insensitive string comparison
+   function. */    
+#if defined(WIN32) || defined(WIN64)
+#define ICSSTRCASECMP _stricmp
+#else
+#define ICSSTRCASECMP strcasecmp
+#endif
+
 
 #ifdef WIN32
 #ifdef BUILD_ICSLIB
@@ -87,6 +100,7 @@ extern "C" {
 /* These determine the sizes of static arrays and strings: */
 #define ICS_MAXDIM 10        /* maximum number of image dimensions.           */
 #define ICS_MAX_LAMBDA 32    /* maximum number of channels.                   */
+#define ICS_MAX_DETECT 32    /* maximum number of detectors.                  */
 #define ICS_STRLEN_TOKEN 32  /* length of a token string.                     */
 #define ICS_STRLEN_OTHER 128 /* length of other strings.                      */
 #define ICS_LINE_LENGTH 1024 /* maximum length of the lines in the .ics file. */
@@ -103,6 +117,8 @@ typedef enum {
     Ics_sint16,    /* integer, signed,   16 bpp */
     Ics_uint32,    /* integer, unsigned, 32 bpp */
     Ics_sint32,    /* integer, signed,   32 bpp */
+    Ics_uint64,    /* integer, unsigned, 64 bpp */
+    Ics_sint64,    /* integer, signed,   64 bpp */
     Ics_real32,    /* real,    signed,   32 bpp */
     Ics_real64,    /* real,    signed,   64 bpp */
     Ics_complex32, /* complex, signed, 2*32 bpp */
@@ -126,6 +142,13 @@ typedef enum {
 } Ics_FileMode;
 
 
+/* Byte orders. */
+typedef enum {
+    IcsByteOrder_littleEndian, /* little endian byte order */
+    IcsByteOrder_bigEndian     /* big endian byte order    */
+} Ics_ByteOrder;
+
+
 /* Structures that define the image representation. They are only used inside
    the ICS data structure. */
 typedef struct {
@@ -153,7 +176,7 @@ typedef struct {
 } Ics_ImelRepresentation;
 
 
-/* A list of sensor parameters that are also equiped with a state. */
+/* A list of sensor parameters that are also equipped with a state. */
 typedef enum {
     ICS_SENSOR_FIRST,
     ICS_SENSOR_IMAGING_DIRECTION,
@@ -170,12 +193,23 @@ typedef enum {
     ICS_SENSOR_PHOTON_COUNT,
     ICS_SENSOR_INTERFACE_PRIMARY,
     ICS_SENSOR_INTERFACE_SECONDARY,
+    ICS_SENSOR_DESCRIPTION,
     
     ICS_SENSOR_DETECTOR_MAGN,
     ICS_SENSOR_DETECTOR_PPU,
     ICS_SENSOR_DETECTOR_BASELINE,
     ICS_SENSOR_DETECTOR_LINE_AVG_COUNT,
 
+    ICS_SENSOR_DETECTOR_OFFSET,
+    ICS_SENSOR_DETECTOR_SENSITIVITY,
+    ICS_SENSOR_DETECTOR_RADIUS,
+    ICS_SENSOR_DETECTOR_SCALE,
+    ICS_SENSOR_DETECTOR_STRETCH,
+    ICS_SENSOR_DETECTOR_ROT,
+    ICS_SENSOR_DETECTOR_MIRROR,
+    ICS_SENSOR_DETECTOR_MODEL,
+    ICS_SENSOR_DETECTOR_REDUCEHIST,
+    
     ICS_SENSOR_STED_DEPLETION_MODE,
     ICS_SENSOR_STED_LAMBDA,
     ICS_SENSOR_STED_SATURATION_FACTOR,
@@ -253,6 +287,8 @@ typedef struct _ICS {
     char                    model[ICS_STRLEN_OTHER];
         /* Number of channels: */
     int                     sensorChannels;
+        /* Number of detectors: */
+    int                     sensorDetectors;
         /* Imaging direction: */
     char                    imagingDirection[ICS_MAX_LAMBDA][ICS_STRLEN_TOKEN];
     Ics_SensorState         imagingDirectionState[ICS_MAX_LAMBDA];
@@ -295,6 +331,9 @@ typedef struct _ICS {
         /* Emission wavelength in nm: */
     double                  interfaceSecondary;
     Ics_SensorState         interfaceSecondaryState;
+        /* Additional per channel description strings: */
+    char                    description[ICS_MAX_LAMBDA][ICS_STRLEN_OTHER];
+    Ics_SensorState         descriptionState[ICS_MAX_LAMBDA];
         /* Excitation beam fill factor: */
     double                  detectorMagn[ICS_MAX_LAMBDA];
     Ics_SensorState         detectorMagnState[ICS_MAX_LAMBDA];
@@ -307,6 +346,33 @@ typedef struct _ICS {
         /* Averaging line count */
     double                  detectorLineAvgCnt[ICS_MAX_LAMBDA];
     Ics_SensorState         detectorLineAvgCntState[ICS_MAX_LAMBDA];
+        /* Detector offsets */
+    double                  detectorOffset[ICS_MAX_LAMBDA][ICS_MAX_DETECT][3];
+    Ics_SensorState         detectorOffsetState[ICS_MAX_LAMBDA];
+        /* Detector sensitivities */
+    double                  detectorSensitivity[ICS_MAX_LAMBDA][ICS_MAX_DETECT];
+    Ics_SensorState         detectorSensitivityState[ICS_MAX_LAMBDA];
+        /* Detector radius */
+    double                  detectorRadius[ICS_MAX_LAMBDA][ICS_MAX_DETECT];
+    Ics_SensorState         detectorRadiusState[ICS_MAX_LAMBDA];
+        /* Detector array scale */
+    double                  detectorScale[ICS_MAX_LAMBDA];
+    Ics_SensorState         detectorScaleState[ICS_MAX_LAMBDA];
+        /* Detector array stretch */
+    double                  detectorStretch[ICS_MAX_LAMBDA];
+    Ics_SensorState         detectorStretchState[ICS_MAX_LAMBDA];
+        /* Detector array rotation */
+    double                  detectorRot[ICS_MAX_LAMBDA];
+    Ics_SensorState         detectorRotState[ICS_MAX_LAMBDA];
+        /* Detector array mirroring */
+    char                    detectorMirror[ICS_MAX_LAMBDA][ICS_STRLEN_TOKEN];
+    Ics_SensorState         detectorMirrorState[ICS_MAX_LAMBDA];
+        /* Detector array model */
+    char                    detectorModel[ICS_MAX_LAMBDA][ICS_STRLEN_TOKEN];
+    Ics_SensorState         detectorModelState[ICS_MAX_LAMBDA];
+        /* Detector reduce hist. */
+    char                    detectorRedHist[ICS_MAX_LAMBDA][ICS_STRLEN_TOKEN];
+    Ics_SensorState         detectorRedHistState[ICS_MAX_LAMBDA];
         /* STED depletion mode: */
     char                    stedDepletionMode[ICS_MAX_LAMBDA][ICS_STRLEN_TOKEN];
     Ics_SensorState         stedDepletionModeState[ICS_MAX_LAMBDA];
@@ -450,13 +516,15 @@ typedef enum {
     IcsErr_NotIcsFile,
         /* The function won't work on the ICS given: */
     IcsErr_NotValidAction,
+        /* Too many detectors specified: */
+    IcsErr_TooManyDetectors,
         /* Too many channels specified: */
     IcsErr_TooManyChans,
         /* Data has too many dimensions: */
     IcsErr_TooManyDims,
         /* Unknown compression type: */
     IcsErr_UnknownCompression,
-         /* The datatype is not recognized: */
+         /* The data type is not recognized: */
     IcsErr_UnknownDataType,
         /* The state is unknown. */
     IcsErr_UnknownSensorState,
@@ -473,11 +541,11 @@ typedef enum {
 } Ics_HistoryWhich;
 
 
-typedef struct {
+typedef struct _Ics_HistoryIterator {
     int  next;                    /* index into history array, pointing to next
                                      string to read, set to -1 if there's no
                                      more to read. */
-    int  previous;                /* index to previous string, useful for relace
+    int  previous;                /* index to previous string, useful for replace
                                      and delete. */
     char key[ICS_STRLEN_TOKEN+1]; /* optional key this iterator looks for. */
 } Ics_HistoryIterator;
@@ -489,7 +557,7 @@ ICSEXPORT const char* IcsGetLibVersion(void);
 
 
 /* Returns 0 if it is not an ICS file, or the version number if it is.  If
-  forcename is non-zero, no extension is appended. */
+  forceName is non-zero, no extension is appended. */
 ICSEXPORT int IcsVersion(const char *filename,
                          int         forceName);
 
@@ -607,6 +675,13 @@ ICSEXPORT Ics_Error IcsSetSource(ICS        *ics,
                                  const char *fname,
                                  size_t      offset);
 
+/* Set the image source byte order for an ICS version 2.0 file. Only valid if
+   writing, and only valid after calling IcsSetSource. If the data type is
+   changed after this call, the byte order information written to file might
+   not be correct. */
+ICSEXPORT Ics_Error IcsSetByteOrder(ICS           *ics,
+                                    Ics_ByteOrder  order);
+
 
 /* Set the compression method and compression parameter. Only valid if
    writing. */
@@ -816,7 +891,7 @@ ICSEXPORT Ics_Error IcsDeleteHistoryStringI(ICS                 *ics,
                                             Ics_HistoryIterator *it);
 
 
-/* Delete last retrieved history line (iterator still points to the same
+/* Replace last retrieved history line (iterator still points to the same
    string). */
 ICSEXPORT Ics_Error IcsReplaceHistoryStringI(ICS                 *ics,
                                              Ics_HistoryIterator *it,
