@@ -58,11 +58,6 @@
 #include "libics_intern.h"
 
 
-#if defined(ICS_DO_GZEXT) && !defined(ICS_ZLIB)
-#undef ICS_DO_GZEXT
-#endif
-
-
 /* Write uncompressed data, with strides. */
 Ics_Error IcsWritePlainWithStrides(const void      *src,
                                    const size_t    *dim,
@@ -87,12 +82,12 @@ Ics_Error IcsWritePlainWithStrides(const void      *src,
             data += (ptrdiff_t)curpos[i] * stride[i] * nBytes;
         }
         if (stride[0] == 1) {
-            if (fwrite(data, nBytes, dim[0], file) != dim[0]) {
+            if (fwrite(data, (size_t)nBytes, dim[0], file) != dim[0]) {
                 return IcsErr_FWriteIds;
             }
         } else {
             for (j = 0; j < dim[0]; j++) {
-                if (fwrite(data, nBytes, 1, file) != 1) {
+                if (fwrite(data, (size_t)nBytes, 1, file) != 1) {
                     return IcsErr_FWriteIds;
                 }
                 data += stride[0] * nBytes;
@@ -149,7 +144,7 @@ Ics_Error IcsWriteIds(const Ics_Header *icsStruct)
                 error = IcsWritePlainWithStrides(icsStruct->data, dim,
                                                  icsStruct->dataStrides,
                                                  icsStruct->dimensions,
-                                                 size, fp);
+                                                 (int)size, fp);
             } else {
                     /* We do the writing in blocks if the data is very large,
                        this avoids a bug in some c library implementations on
@@ -180,7 +175,7 @@ Ics_Error IcsWriteIds(const Ics_Header *icsStruct)
                 error = IcsWriteZipWithStrides(icsStruct->data, dim,
                                                icsStruct->dataStrides,
                                                icsStruct->dimensions,
-                                               size, fp, icsStruct->compLevel);
+                                               (int)size, fp, icsStruct->compLevel);
             } else {
                 error = IcsWriteZip(icsStruct->data, icsStruct->dataLength, fp,
                                     icsStruct->compLevel);
@@ -218,7 +213,7 @@ Ics_Error IcsCopyIds(const char *infilename,
         error = IcsErr_FCopyIds;
         goto exit;
     }
-    if (fseek(in, inoffset, SEEK_SET) != 0) {
+    if (fseek(in, (long)inoffset, SEEK_SET) != 0) {
         error = IcsErr_FCopyIds;
         goto exit;
     }
@@ -282,10 +277,24 @@ static int IcsIsLittleEndianMachine(void)
 
 
 /* Fill the byte order array with the machine's byte order. */
-void IcsFillByteOrder(int bytes,
-                      int machineByteOrder[ICS_MAX_IMEL_SIZE])
+void IcsFillByteOrder(Ics_DataType dataType,
+                      int          bytes,
+                      int          machineByteOrder[ICS_MAX_IMEL_SIZE])
+{
+    if (IcsIsLittleEndianMachine()) {
+        IcsFillLittleEndianByteOrder(dataType, bytes, machineByteOrder);
+    } else {
+        IcsFillBigEndianByteOrder(dataType, bytes, machineByteOrder);
+    }
+}
+
+/* Fill the byte order array with little endian byte order. */
+void IcsFillLittleEndianByteOrder(Ics_DataType dataType,
+                                  int          bytes,
+                                  int          byteOrder[ICS_MAX_IMEL_SIZE])
 {
     int i;
+   (void) dataType; /* Not used, but kept as input for consistency. */
 
 
     if (bytes > ICS_MAX_IMEL_SIZE) {
@@ -294,38 +303,59 @@ void IcsFillByteOrder(int bytes,
         bytes = ICS_MAX_IMEL_SIZE;
     }
 
-    if (IcsIsLittleEndianMachine()) {
-            /* Fill byte order for a little endian machine. */
-        for (i = 0; i < bytes; i++) {
-            machineByteOrder[i] = 1 + i;
+    for (i = 0; i < bytes; i++) {
+        byteOrder[i] = 1 + i;
+    }
+}
+
+/* Fill the byte order array with big endian byte order. */
+void IcsFillBigEndianByteOrder(Ics_DataType dataType,
+                               int          bytes,
+                               int          byteOrder[ICS_MAX_IMEL_SIZE])
+{
+    int i, hbytes;
+
+
+    if (bytes > ICS_MAX_IMEL_SIZE) {
+            /* This will cause problems if undetected, */
+            /* but shouldn't happen anyway */
+        bytes = ICS_MAX_IMEL_SIZE;
+    }
+
+    if (dataType == Ics_complex32 || dataType == Ics_complex64) {
+        hbytes = bytes/2;
+        for (i = 0; i < hbytes; i++) {
+            byteOrder[i]        = hbytes - i;
+            byteOrder[i+hbytes] = bytes - i;
         }
     } else {
-            /* Fill byte order for a big endian machine. */
         for (i = 0; i < bytes; i++) {
-            machineByteOrder[i] = bytes - i;
+            byteOrder[i] = bytes - i;
         }
     }
 }
 
 
 /* Reorder the bytes in the images as specified in the ByteOrder array. */
-static Ics_Error IcsReorderIds(char   *buf,
-                               size_t  length,
-                               int     srcByteOrder[ICS_MAX_IMEL_SIZE],
-                               int     bytes)
+static Ics_Error IcsReorderIds(char        *buf,
+                               size_t       length,
+                               Ics_DataType dataType,
+                               int          srcByteOrder[ICS_MAX_IMEL_SIZE],
+                               int          bytes)
 {
     ICSINIT;
-    int  i, j, imels;
+    int  i;
+    size_t j, imels;
     int  dstByteOrder[ICS_MAX_IMEL_SIZE];
     char imel[ICS_MAX_IMEL_SIZE];
     int  different = 0, empty = 0;
 
 
-    imels = length / bytes;
-    if (length % bytes != 0) return IcsErr_BitsVsSizeConfl;
+    imels = length / (size_t)bytes;
+    if (length % (size_t)bytes != 0) return IcsErr_BitsVsSizeConfl;
 
         /* Create destination byte order: */
-    IcsFillByteOrder(bytes, dstByteOrder);
+    IcsFillByteOrder(dataType, bytes, dstByteOrder);
 
         /* Localize byte order array: */
     for (i = 0; i < bytes; i++){
@@ -336,10 +366,10 @@ static Ics_Error IcsReorderIds(char   *buf,
 
     for (j = 0; j < imels; j++){
         for (i = 0; i < bytes; i++){
-            imel[i] = buf[srcByteOrder[i]-1];
+            imel[srcByteOrder[i]-1] = buf[i];
         }
         for (i = 0; i < bytes; i++){
-            buf[dstByteOrder[i]-1] = imel[i];
+            buf[i] = imel[dstByteOrder[i]-1];
         }
         buf += bytes;
     }
@@ -393,7 +423,7 @@ Ics_Error IcsOpenIds(Ics_Header *icsStruct)
 
     br->dataFilePtr = IcsFOpen(filename, "rb");
     if (br->dataFilePtr == NULL) return IcsErr_FOpenIds;
-    if (fseek(br->dataFilePtr, offset, SEEK_SET) != 0) {
+    if (fseek(br->dataFilePtr, (long)offset, SEEK_SET) != 0) {
         fclose(br->dataFilePtr);
         free(br);
         return IcsErr_FReadIds;
@@ -482,7 +512,8 @@ Ics_Error IcsReadIdsBlock(Ics_Header *icsStruct,
             error = IcsErr_UnknownCompression;
     }
 
-    if (!error) error = IcsReorderIds((char*)dest, n, icsStruct->byteOrder,
+    if (!error) error = IcsReorderIds((char*)dest, n, icsStruct->imel.dataType,
+                                      icsStruct->byteOrder,
                                       IcsGetBytesPerSample(icsStruct));
 
     return error;
@@ -493,7 +524,7 @@ Ics_Error IcsReadIdsBlock(Ics_Header *icsStruct,
 Ics_Error IcsSkipIdsBlock(Ics_Header *icsStruct,
                           size_t      n)
 {
-    return IcsSetIdsBlock (icsStruct, n, SEEK_CUR);
+    return IcsSetIdsBlock (icsStruct, (long)n, SEEK_CUR);
 }
 
 
