@@ -60,14 +60,16 @@
 #include <string.h>
 #include "libics_intern.h"
 
-
-#include "zlib.h"
-
+// Include zlib.h only when available
+#ifdef ICS_ZLIB
+    #include "zlib.h"
+#endif
 
 #define DEF_MEM_LEVEL 8 /* Default value defined in zutil.h */
 
 
 /* GZIP stuff */
+#ifdef ICS_ZLIB
 #ifdef WIN32
 #define OS_CODE 0x0b
 #else
@@ -81,9 +83,10 @@ static int gz_magic[2] = {0x1f, 0x8b}; /* gzip magic header */
 #define ORIG_NAME    0x08 /* bit 3 set: original file name present */
 #define COMMENT      0x10 /* bit 4 set: file comment present */
 #define RESERVED     0xE0 /* bits 5..7: reserved */
-
+#endif // ICS_ZLIB
 
 /* Outputs a long in LSB order to the given stream */
+#ifdef ICS_ZLIB
 static void icsPutLong(FILE *file,
                        unsigned long int x)
 {
@@ -93,9 +96,11 @@ static void icsPutLong(FILE *file,
       x >>= 8;
    }
 }
+#endif
 
 
 /* Reads a long in LSB order from the given stream. */
+#ifdef ICS_ZLIB
 static unsigned long int icsGetLong(FILE *file)
 {
     unsigned long int x = (unsigned long int)getc(file);
@@ -104,6 +109,7 @@ static unsigned long int icsGetLong(FILE *file)
     x += ((unsigned long int)getc(file))<<24;
     return x;
 }
+#endif
 
 
 /* Write ZIP compressed data. This function mostly does:
@@ -163,7 +169,7 @@ Ics_Error IcsWriteZip(const void *inBuf,
     totalCount = 0;
     do {
         if (len - totalCount < ICS_BUF_SIZE) {
-            stream.avail_in = len - totalCount;
+            stream.avail_in = (uInt)(len - totalCount);
         } else {
             stream.avail_in = ICS_BUF_SIZE;
         }
@@ -201,6 +207,10 @@ Ics_Error IcsWriteZip(const void *inBuf,
 
     return err == Z_OK ? IcsErr_Ok : IcsErr_CompressionProblem;
 #else
+    (void)inBuf;
+    (void)len;
+    (void)file;
+    (void)level;
     return IcsErr_UnknownCompression;
 #endif
 }
@@ -235,7 +245,7 @@ Ics_Error IcsWriteZipWithStrides(const void      *src,
     if (outBuf == Z_NULL) return IcsErr_Alloc;
         /* Create an input buffer */
     if (!contiguousLine) {
-        inBuf = (Byte*)malloc(dim[0] * nBytes);
+        inBuf = (Byte*)malloc(dim[0] * (size_t)nBytes);
         if (inBuf == Z_NULL) {
             free(outBuf);
             return IcsErr_Alloc;
@@ -285,14 +295,14 @@ Ics_Error IcsWriteZipWithStrides(const void      *src,
         } else {
             inBuf_ptr = inBuf;
             for (j = 0; j < dim[0]; j++) {
-                memcpy(inBuf_ptr, data, nBytes);
+                memcpy(inBuf_ptr, data, (size_t)nBytes);
                 data += stride[0] * nBytes;
                 inBuf_ptr += nBytes;
             }
         }
             /* Write the compressed data */
         stream.next_in = (Bytef*)inBuf;
-        stream.avail_in = dim[0] * nBytes;
+        stream.avail_in = (uInt)(dim[0] * (size_t)nBytes);
         totalCount += stream.avail_in;
         while (stream.avail_in != 0) {
             if (stream.avail_out == 0) {
@@ -313,7 +323,7 @@ Ics_Error IcsWriteZipWithStrides(const void      *src,
             error = IcsErr_CompressionProblem;
             goto error_exit;
         }
-        crc = crc32(crc, (Bytef*)inBuf, dim[0] * nBytes);
+        crc = crc32(crc, (Bytef*)inBuf, (uInt)(dim[0] * (size_t)nBytes));
             /* This is part of the N-D loop */
         for (i = 1; i < nDims; i++) {
             curPos[i]++;
@@ -365,6 +375,13 @@ Ics_Error IcsWriteZipWithStrides(const void      *src,
         return err == Z_OK ? IcsErr_Ok : IcsErr_CompressionProblem;
     }
 #else
+    (void)src;
+    (void)dim;
+    (void)stride;
+    (void)nDims;
+    (void)nBytes;
+    (void)file;
+    (void)level;
     return IcsErr_UnknownCompression;
 #endif
 }
@@ -396,7 +413,7 @@ Ics_Error IcsOpenZip(Ics_Header *icsStruct)
         len  =  (uInt)getc(file);
         len += ((uInt)getc(file)) << 8;
         if (feof (file)) return IcsErr_CorruptedStream;
-        fseek(file, len, SEEK_CUR);
+        fseek(file, (long)len, SEEK_CUR);
     }
     if ((flags & ORIG_NAME) != 0) {   /* skip the original file name */
         int c;
@@ -449,6 +466,7 @@ Ics_Error IcsOpenZip(Ics_Header *icsStruct)
     br->zlibCRC = crc32(0L, Z_NULL, 0);
     return IcsErr_Ok;
 #else
+    (void)icsStruct;
     return IcsErr_UnknownCompression;
 #endif
 }
@@ -474,6 +492,7 @@ Ics_Error IcsCloseZip(Ics_Header *icsStruct)
     }
     return IcsErr_Ok;
 #else
+    (void)icsStruct;
     return IcsErr_UnknownCompression;
 #endif
 }
@@ -482,8 +501,8 @@ Ics_Error IcsCloseZip(Ics_Header *icsStruct)
 /* Read ZIP compressed data block. This function mostly does:
      gzread((gzFile)br->ZlibStream, outBuf, len); */
 Ics_Error IcsReadZipBlock(Ics_Header *icsStruct,
-                           void       *outBuf,
-                           size_t      len)
+                          void       *outBuf,
+                          size_t      len)
 {
 #ifdef ICS_ZLIB
     Ics_BlockRead *br      = (Ics_BlockRead*)icsStruct->blockRead;
@@ -497,7 +516,7 @@ Ics_Error IcsReadZipBlock(Ics_Header *icsStruct,
 
         /* Read the compressed data */
     do {
-        stream->avail_in = fread(inBuf, 1, ICS_BUF_SIZE, file);
+        stream->avail_in = (uInt)fread(inBuf, 1, ICS_BUF_SIZE, file);
         if (ferror(file)) {
             return IcsErr_FReadIds;
         }
@@ -511,7 +530,7 @@ Ics_Error IcsReadZipBlock(Ics_Header *icsStruct,
                 err = Z_OK;
                 break;
             }
-            bufsize = todo < ICS_BUF_SIZE ? todo : ICS_BUF_SIZE;
+            bufsize = (unsigned int)(todo < ICS_BUF_SIZE ? todo : ICS_BUF_SIZE);
             stream->avail_out = bufsize;
             prevbuf = stream->next_out = (Bytef*)outBuf + len - todo;;
             err = inflate(stream, Z_NO_FLUSH);
@@ -548,6 +567,9 @@ Ics_Error IcsReadZipBlock(Ics_Header *icsStruct,
     if (err == Z_OK) return IcsErr_Ok;
     return IcsErr_DecompressionProblem;
 #else
+    (void)icsStruct;
+    (void)outBuf;
+    (void)len;
     return IcsErr_UnknownCompression;
 #endif
 }
@@ -556,8 +578,8 @@ Ics_Error IcsReadZipBlock(Ics_Header *icsStruct,
 /* Skip ZIP compressed data block. This function mostly does:
      gzseek((gzFile)br->ZlibStream, (z_off_t)offset, whence); */
 Ics_Error IcsSetZipBlock(Ics_Header *icsStruct,
-                          long        offset,
-                          int         whence)
+                         long        offset,
+                         int         whence)
 {
 #ifdef ICS_ZLIB
     ICSINIT;
@@ -567,7 +589,7 @@ Ics_Error IcsSetZipBlock(Ics_Header *icsStruct,
     z_stream*      stream = (z_stream*)br->zlibStream;
 
     if ((whence == SEEK_CUR) && (offset<0)) {
-        offset += stream->total_out;
+        offset += (long)stream->total_out;
         whence = SEEK_SET;
     }
     if (whence == SEEK_SET) {
@@ -579,11 +601,11 @@ Ics_Error IcsSetZipBlock(Ics_Header *icsStruct,
         if (offset==0) return IcsErr_Ok;
     }
 
-    bufsize = offset < ICS_BUF_SIZE ? offset : ICS_BUF_SIZE;
+    bufsize = (unsigned int)(offset < ICS_BUF_SIZE ? offset : ICS_BUF_SIZE);
     buf = malloc(bufsize);
     if (buf == NULL) return IcsErr_Alloc;
 
-    n = offset;
+    n = (size_t)offset;
     while (n > 0) {
         if (n > bufsize) {
             error = IcsReadZipBlock(icsStruct, buf, bufsize);
@@ -601,6 +623,9 @@ Ics_Error IcsSetZipBlock(Ics_Header *icsStruct,
 
     return error;
 #else
+    (void)icsStruct;
+    (void)offset;
+    (void)whence;
     return IcsErr_UnknownCompression;
 #endif
 }
