@@ -1,7 +1,7 @@
 /*
  * libics: Image Cytometry Standard file reading and writing.
  *
- * Copyright 2015-2017:
+ * Copyright 2015-2018:
  *   Scientific Volume Imaging Holding B.V.
  *   Laapersveld 63, 1213 VB Hilversum, The Netherlands
  *   https://www.svi.nl
@@ -107,6 +107,7 @@ typedef enum {
         /* SubsubCategory tokens: */
     ICSTOK_FIRSTSUBSUB,
     ICSTOK_CHANS,
+    ICSTOK_DETECTORS,
     ICSTOK_IMDIR,
     ICSTOK_NUMAPER,
     ICSTOK_OBJQ,
@@ -121,10 +122,20 @@ typedef enum {
     ICSTOK_PHOTCNT,
     ICSTOK_IFACE1,
     ICSTOK_IFACE2,
+    ICSTOK_DESCRIPTION,
     ICSTOK_DETMAG,
     ICSTOK_DETPPU,
     ICSTOK_DETBASELINE,
     ICSTOK_DETLNAVGCNT,
+    ICSTOK_DETOFFSET,
+    ICSTOK_DETSENS,
+    ICSTOK_DETRADIUS,
+    ICSTOK_DETSCALE,
+    ICSTOK_DETSTRETCH,
+    ICSTOK_DETROT,
+    ICSTOK_DETMIRROR,
+    ICSTOK_DETMODEL,
+    ICSTOK_DETREDUCEHIST,
     ICSTOK_STEDDEPLMODE,
     ICSTOK_STEDLAMBDA,
     ICSTOK_STEDSATFACTOR,
@@ -250,8 +261,15 @@ Ics_Error IcsInternAddHistory(Ics_Header *ics,
                               const char *seps);
 
 /* Binary data support functions */
-void IcsFillByteOrder(int bytes,
-                      int machineByteOrder[ICS_MAX_IMEL_SIZE]);
+void IcsFillByteOrder(Ics_DataType dataType,
+                      int          bytes,
+                      int          machineByteOrder[ICS_MAX_IMEL_SIZE]);
+void IcsFillLittleEndianByteOrder(Ics_DataType dataType,
+                                  int          bytes,
+                                  int          byteOrder[ICS_MAX_IMEL_SIZE]);
+void IcsFillBigEndianByteOrder(Ics_DataType dataType,
+                               int          bytes,
+                               int          byteOrder[ICS_MAX_IMEL_SIZE]);
 
 Ics_Error IcsWritePlainWithStrides(const void      *src,
                                    const size_t    *dim,
