@@ -1,7 +1,7 @@
 /*
  * libics: Image Cytometry Standard file reading and writing.
  *
- * Copyright 2015-2017:
+ * Copyright 2015-2019:
  *   Scientific Volume Imaging Holding B.V.
  *   Laapersveld 63, 1213 VB Hilversum, The Netherlands
  *   https://www.svi.nl
@@ -44,6 +44,12 @@
 #include <string.h>
 #include "libics_intern.h"
 
+/* Use thread safe string tokenization if available */
+#ifdef HAVE_STRTOK_R
+#define STRTOK(str, seps) strtok_r(str, seps, &saveptr)
+#else
+#define STRTOK(str, seps) strtok(str, seps)
+#endif
 
 /* Find the index for "bits", which should be the first parameter. */
 static int icsGetBitsParam(char order[ICS_MAXDIM+1][ICS_STRLEN_TOKEN],
@@ -146,8 +152,8 @@ static Ics_Error getIcsSeparators(FILE *fi,
             }
         }
     }
-    seps[0] = sep1;
-    seps[1] = sep2;
+    seps[0] = (char)sep1;
+    seps[1] = (char)sep2;
     seps[2] = '\0';
 
     return IcsErr_Ok;
@@ -161,14 +167,17 @@ static Ics_Error getIcsVersion(FILE       *fi,
     ICSINIT;
     char *word;
     char  line[ICS_LINE_LENGTH];
+#ifdef HAVE_STRTOK_R
+    char *saveptr;
+#endif
 
 
     if (icsFGetStr(line, ICS_LINE_LENGTH, fi, seps[1]) == NULL)
         return IcsErr_FReadIcs;
-    word = strtok(line, seps);
+    word = STRTOK(line, seps);
     if (word == NULL) return IcsErr_NotIcsFile;
     if (strcmp(word, ICS_VERSION) != 0) return IcsErr_NotIcsFile;
-    word = strtok(NULL, seps);
+    word = STRTOK(NULL, seps);
     if (word == NULL) return IcsErr_NotIcsFile;
     if (strcmp(word, "1.0") == 0) {
         *ver = 1;
@@ -188,11 +197,14 @@ static Ics_Error getIcsFileName(FILE       *fi,
     ICSINIT;
     char *word;
     char  line[ICS_LINE_LENGTH];
+#ifdef HAVE_STRTOK_R
+    char *saveptr;
+#endif
 
 
     if (icsFGetStr(line, ICS_LINE_LENGTH, fi, seps[1]) == NULL)
         return IcsErr_FReadIcs;
-    word = strtok(line, seps);
+    word = STRTOK(line, seps);
     if (word == NULL) return IcsErr_NotIcsFile;
     if (strcmp(word, ICS_FILENAME) != 0) return IcsErr_NotIcsFile;
 
@@ -203,13 +215,15 @@ static Ics_Error getIcsFileName(FILE       *fi,
 static Ics_Token getIcsToken(char           *str,
                              Ics_SymbolList *listSpec)
 {
-    int i;
+    int       i;
     Ics_Token token = ICSTOK_NONE;
 
-
+        /* Because some older ics versions have uncapitalized subsubcat
+           symbols (e.g. "channels" instead of the current "Channels"), do a
+           case insenstive string comparison for backward compatiblity. */
     if (str != NULL) {
         for (i = 0; i < listSpec->entries; i++) {
-            if (strcmp(listSpec->list[i].name, str) == 0) {
+            if (ICSSTRCASECMP(listSpec->list[i].name, str) == 0) {
                 token = listSpec->list[i].token;
             }
         }
@@ -224,31 +238,44 @@ static Ics_Error getIcsCat(char        *str,
                            Ics_Token   *cat,
                            Ics_Token   *subCat,
                            Ics_Token   *subSubCat,
-                           const char **index)
+                           const char **index1,
+                           const char **index2)
 {
     ICSINIT;
-    char *token, buffer[ICS_LINE_LENGTH], *idx;
+    char *token, buffer[ICS_LINE_LENGTH], *idx1, *idx2;
+#ifdef HAVE_STRTOK_R
+    char *saveptr;
+#endif
 
 
     *subCat = *subSubCat = ICSTOK_NONE;
-    *index = NULL;
+    *index1 = NULL;
+    *index2 = NULL;
 
     IcsStrCpy(buffer, str, ICS_LINE_LENGTH);
-    token = strtok(buffer, seps);
+    token = STRTOK(buffer, seps);
     *cat = getIcsToken(token, &G_Categories);
     if (*cat == ICSTOK_NONE) return IcsErr_MissCat;
     if ((*cat != ICSTOK_HISTORY) &&(*cat != ICSTOK_END)) {
-        token = strtok(NULL, seps);
+        token = STRTOK(NULL, seps);
         *subCat = getIcsToken(token, &G_SubCategories);
         if (*subCat == ICSTOK_NONE) return IcsErr_MissSubCat;
         if (*subCat == ICSTOK_SPARAMS || *subCat == ICSTOK_SSTATES) {
-            token = strtok(NULL, seps);
+            token = STRTOK(NULL, seps);
             if (token[strlen(token) - 1] == ']') {
-                idx = strchr(token, '[');
-                if (idx) {
-                    token[strlen(token) - 1] = '\0';
-                    *idx = '\0';
-                    *index = idx + 1;
+                idx1 = strchr(token, '[');
+                if (idx1) {
+                    idx2 = strchr(idx1 + 1, '[');
+                }
+                if (idx1) {
+                        /* Todo: Check that this line is indeed not
+                            //necessary. */
+                        /* token[strlen(token) - 1] = '\0'; */
+                    *idx1 = '\0';
+                    *index1 = idx1 + 1;
+                }
+                if (idx2) {
+                    *index2 = idx2 + 1;
                 }
             }
             *subSubCat = getIcsToken(token, &G_SubSubCategories);
@@ -257,10 +284,10 @@ static Ics_Error getIcsCat(char        *str,
     }
 
         /* Copy the remaining stuff into 'str' */
-    if ((token = strtok(NULL, seps)) != NULL) {
+    if ((token = STRTOK(NULL, seps)) != NULL) {
         strcpy(str, token);
     }
-    while ((token = strtok(NULL, seps)) != NULL) {
+    while ((token = STRTOK(NULL, seps)) != NULL) {
         IcsAppendChar(str, seps[0]);
         strcat(str, token);
     }
@@ -301,7 +328,7 @@ do {                                            \
     while (ptr != NULL && i < ICS_MAX_LAMBDA) { \
         IcsStrCpy(icsStruct->FIELD[i++],        \
                   ptr, ICS_STRLEN_TOKEN);       \
-        ptr = strtok(NULL, seps);               \
+        ptr = STRTOK(NULL, seps);               \
     }                                           \
 } while (0)
 
@@ -318,7 +345,7 @@ do {                                            \
 do {                                            \
     while (ptr != NULL && i < ICS_MAX_LAMBDA) { \
         icsStruct->FIELD[i++] = atoi(ptr);      \
-        ptr = strtok(NULL, seps);               \
+        ptr = STRTOK(NULL, seps);               \
     }                                           \
 } while (0)
 
@@ -327,7 +354,7 @@ do {                                            \
 do {                                            \
     while (ptr != NULL && i < ICS_MAX_LAMBDA) { \
         icsStruct->FIELD[i++] = atof(ptr);      \
-        ptr = strtok(NULL, seps);               \
+        ptr = STRTOK(NULL, seps);               \
     }                                           \
 } while (0)
 
@@ -337,7 +364,7 @@ do {                                            \
     while (ptr != NULL && i < ICS_MAX_LAMBDA) { \
         error = getIcsSensorState(ptr, &state); \
         icsStruct->FIELD ## State[i++] = state; \
-        ptr = strtok(NULL, seps);               \
+        ptr = STRTOK(NULL, seps);               \
     }                                           \
 } while(0)
 
@@ -359,11 +386,14 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
     ICSINIT;
     ICS_INIT_LOCALE;
     FILE            *fp;
-    int              end        = 0, i, j, bits;
+    int              end        = 0, si, sj;
+    size_t           i, j;
     char             seps[3], *ptr, *data;
     char             line[ICS_LINE_LENGTH];
     Ics_Token        cat, subCat, subSubCat;
-    const char      *idx;
+    int              detID;
+    const char      *idx1;
+    const char      *idx2;
         /* These are temporary buffers to hold the data read until it is copied
            to the Ics_Header structure. This is needed because the Ics_Header
            structure is made to look more like we like to see images, compared
@@ -378,6 +408,9 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
     char             label[ICS_MAXDIM+1][ICS_STRLEN_TOKEN];
     char             unit[ICS_MAXDIM+1][ICS_STRLEN_TOKEN];
     Ics_SensorState  state      = IcsSensorState_default;
+#ifdef HAVE_STRTOK_R
+    char *saveptr;
+#endif
 
 
     for (i = 0; i < ICS_MAXDIM+1; i++) {
@@ -404,12 +437,13 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
 
     if (!error) error = getIcsVersion(fp, seps, &(icsStruct->version));
     if (!error) error = getIcsFileName(fp, seps);
-
+    
     while (!end && !error
            && (icsFGetStr(line, ICS_LINE_LENGTH, fp, seps[1]) != NULL)) {
-        if (getIcsCat(line, seps, &cat, &subCat, &subSubCat, &idx) != IcsErr_Ok)
+        if (getIcsCat(line, seps, &cat, &subCat, &subSubCat, &idx1, &idx2) !=
+            IcsErr_Ok)
             continue;
-        ptr = strtok(line, seps);
+        ptr = STRTOK(line, seps);
         i = 0;
         switch (cat) {
             case ICSTOK_END:
@@ -449,13 +483,13 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                     case ICSTOK_ORDER:
                         while (ptr!= NULL && i < ICS_MAXDIM+1) {
                             IcsStrCpy(order[i++], ptr, ICS_STRLEN_TOKEN);
-                            ptr = strtok(NULL, seps);
+                            ptr = STRTOK(NULL, seps);
                         }
                         break;
                     case ICSTOK_SIZES:
                         while (ptr!= NULL && i < ICS_MAXDIM+1) {
                             sizes[i++] = IcsStrToSize(ptr);
-                            ptr = strtok(NULL, seps);
+                            ptr = STRTOK(NULL, seps);
                         }
                         break;
                     case ICSTOK_COORD:
@@ -528,7 +562,7 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                     case ICSTOK_BYTEO:
                         while (ptr!= NULL && i < ICS_MAX_IMEL_SIZE) {
                             icsStruct->byteOrder[i++] = atoi(ptr);
-                            ptr = strtok(NULL, seps);
+                            ptr = STRTOK(NULL, seps);
                         }
                         break;
                     default:
@@ -541,25 +575,25 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                     case ICSTOK_ORIGIN:
                         while (ptr!= NULL && i < ICS_MAXDIM+1) {
                             origin[i++] = atof(ptr);
-                            ptr = strtok(NULL, seps);
+                            ptr = STRTOK(NULL, seps);
                         }
                         break;
                     case ICSTOK_SCALE:
                         while (ptr!= NULL && i < ICS_MAXDIM+1) {
                             scale[i++] = atof(ptr);
-                            ptr = strtok(NULL, seps);
+                            ptr = STRTOK(NULL, seps);
                         }
                         break;
                     case ICSTOK_UNITS:
                         while (ptr!= NULL && i < ICS_MAXDIM+1) {
                             IcsStrCpy(unit[i++], ptr, ICS_STRLEN_TOKEN);
-                            ptr = strtok(NULL, seps);
+                            ptr = STRTOK(NULL, seps);
                         }
                         break;
                     case ICSTOK_LABELS:
                         while (ptr!= NULL && i < ICS_MAXDIM+1) {
                             IcsStrCpy(label[i++], ptr, ICS_STRLEN_TOKEN);
-                            ptr = strtok(NULL, seps);
+                            ptr = STRTOK(NULL, seps);
                         }
                         break;
                     default:
@@ -568,7 +602,7 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                 break;
             case ICSTOK_HISTORY:
                 if (ptr != NULL) {
-                    data = strtok(NULL, seps+1); /* This will get the rest of
+                    data = STRTOK(NULL, seps+1); /* This will get the rest of
                                                     the line */
                     if (data == NULL) { /* data is not allowed to be "", but ptr
                                            is */
@@ -595,7 +629,7 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                         while (ptr != NULL && i < ICS_MAX_LAMBDA) {
                             IcsStrCpy(icsStruct->type[i++], ptr,
                                       ICS_STRLEN_TOKEN);
-                            ptr = strtok(NULL, seps);
+                            ptr = STRTOK(NULL, seps);
                         }
                         break;
                     case ICSTOK_MODEL:
@@ -610,7 +644,16 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                                     int v = atoi(ptr);
                                     icsStruct->sensorChannels = v;
                                     if (v > ICS_MAX_LAMBDA) {
-                                        error = IcsErr_TooManyChans;
+                                        error = IcsErr_TooManyChans; 
+                                    }
+                                }
+                                break;
+                            case ICSTOK_DETECTORS:
+                                if (ptr != NULL) {
+                                    int v = atoi(ptr);
+                                    icsStruct->sensorDetectors = v;
+                                    if (v > ICS_MAX_DETECT) {
+                                        error = IcsErr_TooManyDetectors; 
                                     }
                                 }
                                 break;
@@ -656,6 +699,9 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                             case ICSTOK_IFACE2:
                                 ICS_SET_SENSOR_DOUBLE_ONE(interfaceSecondary);
                                 break;
+                            case ICSTOK_DESCRIPTION:
+                                ICS_SET_SENSOR_STRING(description);
+                                break;
                             case ICSTOK_DETMAG:
                                 ICS_SET_SENSOR_DOUBLE(detectorMagn);
                                 break;
@@ -668,6 +714,85 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                             case ICSTOK_DETLNAVGCNT:
                                 ICS_SET_SENSOR_DOUBLE(detectorLineAvgCnt);
                                 break;
+                            case ICSTOK_DETOFFSET:
+                                while (ptr != NULL && i < ICS_MAX_LAMBDA) {
+                                    detID = atoi(idx1);
+                                    switch (idx2[0]) {
+                                        case  'X':
+                                            icsStruct->
+                                                detectorOffset[i++][detID][0]
+                                                = atof(ptr);
+                                            break;
+                                        case  'Y':
+                                            icsStruct->
+                                                detectorOffset[i++][detID][1]
+                                                = atof(ptr);
+                                            break;
+                                        case  'Z':
+                                            icsStruct->
+                                                detectorOffset[i++][detID][2]
+                                                = atof(ptr);
+                                            break;
+                                        default:
+                                            break;
+                                    }
+                                    ptr = STRTOK(NULL, seps);
+                                }
+                                break;
+                            case ICSTOK_DETSENS:
+                                while (ptr != NULL && i < ICS_MAX_LAMBDA) {
+                                    detID = atoi(idx1);
+                                    icsStruct->detectorSensitivity[i++][detID]
+                                        = atof(ptr);
+                                    ptr = STRTOK(NULL, seps);
+                                }
+                                break;
+                            case ICSTOK_DETRADIUS:
+                                    /* This is a temporary fix to provide
+                                       support for ics files with a non-vector
+                                       detector radius. */
+                                    /* Todo: Remove this in due time. */
+                                if (idx1 == NULL) {
+                                    printf("Using non-vector detRadius.\n");
+                                    if (ptr != NULL) {
+                                        printf("Filling vector with single "
+                                               "value.\n");
+                                        int k;
+                                        for (i = 0; i < ICS_MAX_LAMBDA; i++) {
+                                            for (k = 0; k < ICS_MAX_DETECT;
+                                                 k++) {
+                                                icsStruct->detectorRadius[i][k]
+                                                    = atof(ptr);
+                                            }
+                                        }
+                                    }
+                                    break;
+                                }
+                                while (ptr != NULL && i < ICS_MAX_LAMBDA) {
+                                    detID = atoi(idx1);
+                                    icsStruct->detectorRadius[i++][detID]
+                                        = atof(ptr);
+                                    ptr = STRTOK(NULL, seps);
+                                }
+                                break;
+                            case ICSTOK_DETSCALE:
+                                ICS_SET_SENSOR_DOUBLE(detectorScale);
+                                break;
+                            case ICSTOK_DETSTRETCH:
+                                ICS_SET_SENSOR_DOUBLE(detectorStretch);
+                                break;
+                            case ICSTOK_DETROT:
+                                ICS_SET_SENSOR_DOUBLE(detectorRot);
+                                break;
+                            case ICSTOK_DETMIRROR:
+                                ICS_SET_SENSOR_STRING(detectorMirror);
+                                break;
+                            case ICSTOK_DETMODEL:
+                                ICS_SET_SENSOR_STRING(detectorModel);
+                                break;
+                            case ICSTOK_DETREDUCEHIST:
+                                ICS_SET_SENSOR_STRING(detectorRedHist);
+                                break;
                             case ICSTOK_STEDDEPLMODE:
                                 ICS_SET_SENSOR_STRING(stedDepletionMode);
                                 break;
@@ -697,7 +822,7 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                                 break;
                             case ICSTOK_SPIMPLANEPROPDIR:
                                 while (ptr != NULL && i < ICS_MAX_LAMBDA) {
-                                    switch (idx[0]) {
+                                    switch (idx1[0]) {
                                         case  'X':
                                             icsStruct->spimPlanePropDir[i++][0]
                                                 = atof(ptr);
@@ -713,7 +838,7 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                                         default:
                                             break;
                                     }
-                                    ptr = strtok(NULL, seps);
+                                    ptr = STRTOK(NULL, seps);
                                 }
                                 break;
                             case ICSTOK_SPIMPLANECENTEROFF:
@@ -782,6 +907,9 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                             case ICSTOK_IFACE2:
                                 ICS_SET_SENSOR_STATE_ONE(interfaceSecondary);
                                 break;
+                            case ICSTOK_DESCRIPTION:
+                                ICS_SET_SENSOR_STATE(description);
+                                break;
                             case ICSTOK_DETMAG:
                                 ICS_SET_SENSOR_STATE(detectorMagn);
                                 break;
@@ -794,6 +922,33 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
                             case ICSTOK_DETLNAVGCNT:
                                 ICS_SET_SENSOR_STATE(detectorLineAvgCnt);
                                 break;
+                            case ICSTOK_DETOFFSET:
+                                ICS_SET_SENSOR_STATE(detectorOffset);
+                                break;
+                            case ICSTOK_DETSENS:
+                                ICS_SET_SENSOR_STATE(detectorSensitivity);
+                                break;
+                            case ICSTOK_DETRADIUS:
+                                ICS_SET_SENSOR_STATE(detectorRadius);
+                                break;
+                            case ICSTOK_DETSCALE:
+                                ICS_SET_SENSOR_STATE(detectorScale);
+                                break;
+                            case ICSTOK_DETSTRETCH:
+                                ICS_SET_SENSOR_STATE(detectorStretch);
+                                break;
+                            case ICSTOK_DETROT:
+                                ICS_SET_SENSOR_STATE(detectorRot);
+                                break;
+                            case ICSTOK_DETMIRROR:
+                                ICS_SET_SENSOR_STATE(detectorMirror);
+                                break;
+                            case ICSTOK_DETMODEL:
+                                ICS_SET_SENSOR_STATE(detectorModel);
+                                break;
+                            case ICSTOK_DETREDUCEHIST:
+                                ICS_SET_SENSOR_STATE(detectorRedHist);
+                                break;
                             case ICSTOK_STEDDEPLMODE:
                                 ICS_SET_SENSOR_STATE(stedDepletionMode);
                                 break;
@@ -861,33 +1016,33 @@ Ics_Error IcsReadIcs(Ics_Header *icsStruct,
            files in which a single microscope type is defined and multiple
            sensor channels, the microscope type will be duplicated to all sensor
            channels. */
-    for (j = 1; j < icsStruct->sensorChannels; j++) {
-        if (strlen(icsStruct->type[j]) == 0) {
-            IcsStrCpy(icsStruct->type[j], icsStruct->type[0],
+    for (sj = 1; sj < icsStruct->sensorChannels; sj++) {
+        if (strlen(icsStruct->type[sj]) == 0) {
+            IcsStrCpy(icsStruct->type[sj], icsStruct->type[0],
                        ICS_STRLEN_TOKEN);
         }
     }
 
     if (!error) {
-        bits = icsGetBitsParam(order, parameters);
+        int bits = icsGetBitsParam(order, parameters);
         if (bits < 0) {
             error = IcsErr_MissBits;
         } else {
             IcsGetDataTypeProps(&(icsStruct->imel.dataType), format, sign,
                                 sizes[bits]);
-            for (j = 0, i = 0; i < parameters; i++) {
-                if (i == bits) {
-                    icsStruct->imel.origin = origin[i];
+            for (sj = 0, si = 0; si < parameters; si++) {
+                if (si == bits) {
+                    icsStruct->imel.origin = origin[si];
                     icsStruct->imel.scale = scale[i];
-                    strcpy(icsStruct->imel.unit, unit[i]);
+                    strcpy(icsStruct->imel.unit, unit[si]);
                 } else {
-                    icsStruct->dim[j].size = sizes[i];
-                    icsStruct->dim[j].origin = origin[i];
-                    icsStruct->dim[j].scale = scale[i];
-                    strcpy(icsStruct->dim[j].order, order[i]);
-                    strcpy(icsStruct->dim[j].label, label[i]);
-                    strcpy(icsStruct->dim[j].unit, unit[i]);
-                    j++;
+                    icsStruct->dim[sj].size = sizes[si];
+                    icsStruct->dim[sj].origin = origin[si];
+                    icsStruct->dim[sj].scale = scale[si];
+                    strcpy(icsStruct->dim[sj].order, order[si]);
+                    strcpy(icsStruct->dim[sj].label, label[si]);
+                    strcpy(icsStruct->dim[sj].unit, unit[si]);
+                    sj++;
                 }
             }
             icsStruct->dimensions = parameters - 1;
