@@ -1,7 +1,7 @@
 /*
  * libics: Image Cytometry Standard file reading and writing.
  *
- * Copyright 2015-2017:
+ * Copyright 2015-2018:
  *   Scientific Volume Imaging Holding B.V.
  *   Laapersveld 63, 1213 VB Hilversum, The Netherlands
  *   https://www.svi.nl
@@ -127,8 +127,8 @@ void IcsStrCpy(char       *dest,
                int         len)
 {
     if (dest != src) {
-        int nchar = strlen(src);
-        nchar =(nchar > len-1) ? len-1 : nchar;
+        size_t nchar = strlen(src);
+        nchar = (nchar > (size_t)len-1) ? (size_t)len-1 : nchar;
         memcpy(dest, src, nchar);
         dest[nchar] = 0;
     }
@@ -139,7 +139,7 @@ void IcsStrCpy(char       *dest,
 void IcsAppendChar(char *line,
                    char  ch)
 {
-    int len = strlen(line);
+    size_t len = strlen(line);
     line[len] = ch;
     line[len+1] = '\0';
 }
@@ -176,7 +176,7 @@ static char *IcsFileNameFind(const char *str)
  extension could be found. */
 char *IcsExtensionFind(const char *str)
 {
-    int         len;
+    size_t     len;
     const char *ext;
 
 
@@ -321,7 +321,7 @@ Ics_Error IcsOpenIcs(FILE **fpp,
 /* Initialize the Ics_Header structure. */
 void IcsInit(Ics_Header *icsStruct)
 {
-    int i;
+    int i, j;
 
     icsStruct->version = 2; /* We write an ICS v.2.0 as default */
     icsStruct->fileMode = IcsFileMode_write;
@@ -369,6 +369,7 @@ void IcsInit(Ics_Header *icsStruct)
     icsStruct->interfaceSecondary = 0.0;
     icsStruct->interfaceSecondaryState = IcsSensorState_default;
     icsStruct->sensorChannels = 0;
+    icsStruct->sensorDetectors = 0;
     for (i = 0; i < ICS_MAX_LAMBDA; i++) {
         icsStruct->type[i][0] = '\0';
         icsStruct->imagingDirection[i][0] = '\0';
@@ -387,6 +388,8 @@ void IcsInit(Ics_Header *icsStruct)
         icsStruct->lambdaEmState[i] = IcsSensorState_default;
         icsStruct->exPhotonCnt[i] = 1;
         icsStruct->exPhotonCntState[i] = IcsSensorState_default;
+        icsStruct->description[i][0] = '\0';
+        icsStruct->descriptionState[i] = IcsSensorState_default;
         icsStruct->detectorMagn[i] = 1.0;
         icsStruct->detectorMagnState[i] = IcsSensorState_default;
         icsStruct->detectorPPU[i] = 1.0;
@@ -395,6 +398,33 @@ void IcsInit(Ics_Header *icsStruct)
         icsStruct->detectorBaselineState[i] = IcsSensorState_default;
         icsStruct->detectorLineAvgCnt[i] = 1.0;
         icsStruct->detectorLineAvgCntState[i] = IcsSensorState_default;
+            /* The per detector parameters must be initialized for each
+               channel for each detector. */
+        for (j = 0; j < ICS_MAX_DETECT; j++) {
+            icsStruct->detectorOffset[i][j][0]   = 0.0;
+            icsStruct->detectorOffset[i][j][1]   = 0.0;
+            icsStruct->detectorOffset[i][j][2]   = 0.0;
+            icsStruct->detectorSensitivity[i][j] = 1.0;
+            icsStruct->detectorRadius[i][j]      = 0.0;
+        }
+
+        icsStruct->detectorScale[i] = 1.0;
+        icsStruct->detectorScaleState[i] = IcsSensorState_default;
+        icsStruct->detectorStretch[i] = 1.0;
+        icsStruct->detectorStretchState[i] = IcsSensorState_default;
+        icsStruct->detectorRot[i] = 0.0;
+        icsStruct->detectorRotState[i] = IcsSensorState_default;
+        icsStruct->detectorMirror[i][0] = 'N';
+        icsStruct->detectorMirror[i][1] = '\0';
+        icsStruct->detectorMirrorState[i] = IcsSensorState_default;
+        icsStruct->detectorModel[i][0] = '\0';
+        icsStruct->detectorModelState[i] = IcsSensorState_default;
+        icsStruct->detectorRedHist[i][0] = '\0';
+        icsStruct->detectorRedHistState[i] = IcsSensorState_default;
+        
+        icsStruct->detectorOffsetState[i] = IcsSensorState_default;
+        icsStruct->detectorSensitivityState[i] = IcsSensorState_default;
+        icsStruct->detectorRadiusState[i] = IcsSensorState_default;
         icsStruct->stedDepletionMode[i][0] = '\0';
         icsStruct->stedDepletionModeState[i] = IcsSensorState_default;
         icsStruct->stedLambda[i] = 0.0;
@@ -437,7 +467,7 @@ void IcsInit(Ics_Header *icsStruct)
 /* Find the number of bytes per sample. */
 int IcsGetBytesPerSample(const Ics_Header *icsStruct)
 {
-    return IcsGetDataTypeSize(icsStruct->imel.dataType);
+    return (int)IcsGetDataTypeSize(icsStruct->imel.dataType);
 }
 
 
@@ -461,6 +491,8 @@ size_t IcsGetDataTypeSize(Ics_DataType dataType)
         case Ics_real32:
             bytes = 4;
             break;
+        case Ics_uint64:
+        case Ics_sint64:
         case Ics_real64:
         case Ics_complex32:
             bytes = 8;
@@ -488,10 +520,13 @@ void IcsGetPropsDataType(Ics_DataType  dataType,
         case Ics_uint8:
         case Ics_uint16:
         case Ics_uint32:
+        case Ics_uint64:
             *sign = 0;
+            /* fallthrough */
         case Ics_sint8:
         case Ics_sint16:
         case Ics_sint32:
+        case Ics_sint64:
             *format = IcsForm_integer;
             break;
         case Ics_real32:
@@ -526,6 +561,9 @@ void IcsGetDataTypeProps(Ics_DataType* dataType,
                 case 32:
                     *dataType = sign ? Ics_sint32 : Ics_uint32;
                     break;
+                case 64:
+                    *dataType = sign ? Ics_sint64 : Ics_uint64;
+                    break;
                 default:
                     *dataType = Ics_unknown;
             }
